package sun.tools.jar;

import java.io.PrintStream;
import java.io.OutputStream;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;
import java.security.MessageDigest;
import java.io.FileInputStream;
import java.io.File;
import java.util.Enumeration;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.util.Hashtable;
import sun.net.www.MessageHeader;
import java.util.Vector;

public class Manifest
{
    private Vector<MessageHeader> entries;
    private byte[] tmpbuf;
    private Hashtable<String, MessageHeader> tableEntries;
    static final String[] hashes;
    static final byte[] EOL;
    static final boolean debug = false;
    static final String VERSION = "1.0";
    
    static final void debug(String s) {
    }
    
    public Manifest() {
        this.entries = new Vector();
        this.tmpbuf = new byte[512];
        this.tableEntries = new Hashtable();
    }
    
    public Manifest(byte[] buf) throws IOException {
        this(new ByteArrayInputStream(buf), false);
    }
    
    public Manifest(InputStream inputStream) throws IOException {
        this(inputStream, true);
    }
    
    public Manifest(InputStream inputStream, boolean b) throws IOException {
        this.entries = new Vector();
        this.tmpbuf = new byte[512];
        this.tableEntries = new Hashtable();
        if (!inputStream.markSupported()) {
            inputStream = new BufferedInputStream(inputStream);
        }
        while (true) {
            inputStream.mark(1);
            if (inputStream.read() == -1) {
                break;
            }
            inputStream.reset();
            MessageHeader o = new MessageHeader(inputStream);
            if (b) {
                this.doHashes(o);
            }
            this.addEntry(o);
        }
    }
    
    public Manifest(String[] array) throws IOException {
        this.entries = new Vector();
        this.tmpbuf = new byte[512];
        this.tableEntries = new Hashtable();
        MessageHeader messageHeader = new MessageHeader();
        messageHeader.add("Manifest-Version", "1.0");
        messageHeader.add("Created-By", "Manifest JDK " + System.getProperty("java.version"));
        this.addEntry(messageHeader);
        this.addFiles(null, array);
    }
    
    public void addEntry(MessageHeader messageHeader) {
        this.entries.addElement(messageHeader);
        String value = messageHeader.findValue("Name");
        debug("addEntry for name: " + value);
        if (value != null) {
            this.tableEntries.put(value, messageHeader);
        }
    }
    
    public MessageHeader getEntry(String key) {
        return (MessageHeader)this.tableEntries.get(key);
    }
    
    public MessageHeader entryAt(int index) {
        return (MessageHeader)this.entries.elementAt(index);
    }
    
    public Enumeration<MessageHeader> entries() {
        return this.entries.elements();
    }
    
    public void addFiles(File parent, String[] array) throws IOException {
        if (array == null) {
            return;
        }
        for (int i = 0; i < array.length; ++i) {
            Object o;
            if (parent == null) {
                o = new File(array[i]);
            }
            else {
                o = new File(parent, array[i]);
            }
            if (o.isDirectory()) {
                this.addFiles(o, o.list());
            }
            else {
                this.addFile(o);
            }
        }
        return;
    }
    
    private final String stdToLocal(String s) {
        return s.replace('/', File.separatorChar);
    }
    
    private final String localToStd(String s) {
        s = s.replace(File.separatorChar, '/');
        if (s.startsWith("./")) {
            s = s.substring(2);
        }
        else if (s.startsWith("/")) {
            s = s.substring(1);
        }
        return s;
    }
    
    public void addFile(File file) throws IOException {
        String localToStd = this.localToStd(file.getPath());
        if (this.tableEntries.get(localToStd) == null) {
            MessageHeader messageHeader = new MessageHeader();
            messageHeader.add("Name", localToStd);
            this.addEntry(messageHeader);
        }
    }
    
    public void doHashes(MessageHeader messageHeader) throws IOException {
        String value = messageHeader.findValue("Name");
        if (value == null || value.endsWith("/")) {
            return;
        }
        for (int i = 0; i < Manifest.hashes.length; ++i) {
            FileInputStream o = new FileInputStream(this.stdToLocal(value));
            try {
                MessageDigest instance = MessageDigest.getInstance(Manifest.hashes[i]);
                int read;
                while ((read = o.read(this.tmpbuf, 0, this.tmpbuf.length)) != -1) {
                    instance.update(this.tmpbuf, 0, read);
                }
                messageHeader.set(Manifest.hashes[i] + "-Digest", Base64.getMimeEncoder().encodeToString(instance.digest()));
            }
            catch (NoSuchAlgorithmException ex) {
                throw new JarException("Digest algorithm " + Manifest.hashes[i] + " not available.");
            }
            finally {
                o.close();
            }
        }
        return;
    }
    
    public void stream(OutputStream out) throws IOException {
        PrintStream printStream;
        if ((out instanceof PrintStream)) {
            printStream = (PrintStream)out;
        }
        else {
            printStream = new PrintStream(out);
        }
        MessageHeader messageHeader = (MessageHeader)this.entries.elementAt(0);
        if (messageHeader.findValue("Manifest-Version") == null) {
            String property = System.getProperty("java.version");
            if (messageHeader.findValue("Name") == null) {
                messageHeader.prepend("Manifest-Version", "1.0");
                messageHeader.add("Created-By", "Manifest JDK " + property);
            }
            else {
                printStream.print("Manifest-Version: 1.0\r\nCreated-By: " + property + "\r\n\r\n");
            }
            printStream.flush();
        }
        messageHeader.print(printStream);
        for (int i = 1; i < this.entries.size(); ++i) {
            ((MessageHeader)this.entries.elementAt(i)).print(printStream);
        }
    }
    
    public static boolean isManifestName(String s) {
        if (s.charAt(0) == '/') {
            s = s.substring(1, s.length());
        }
        s = s.toUpperCase();
        return s.equals("META-INF/MANIFEST.MF");
    }
    
    static {
        hashes = new String[] { "SHA" };
        EOL = new byte[] { 13, 10 };
    }
}
